// Now let's make our dots do something useful: they'll
// serve as the handles for a Bezier curve.  The basic idea
// is the same (move four dots around).  But the view is a 
// bit more complicated, since it's really the curve that we
// want to see.

float[] hxs;
float[] hys;

int which_drag;

// Smaller dots, since they're not the focus.
float radius = 10.0;

void setup()
{
  size( 500, 500 );
  hxs = new float[] { 50.0, 150.0, 350.0, 450.0 };
  hys = new float[] { 300.0, 200.0, 200.0, 300.0 };

  which_drag = -1;
}

void draw()
{
  background( 220 );
  
  // Draw the Bezier curve.  Note that we're doing this
  // *first*.  That's because the handles should be drawn
  // on top of the curve!
  stroke( 0 );
  strokeWeight( 3.0 );
  noFill();
  bezier( hxs[0], hys[0], hxs[1], hys[1], 
    hxs[2], hys[2], hxs[3], hys[3] );
    
  // For good measure, draw line segments from Point 0
  // to Point 1, and from Point 2 to Point 3.  This
  // mimics Illustrator to some extent -- the line 
  // segments are the tangents at the start and end
  // of the curve.
  stroke( 128 );
  strokeWeight( 1.0 );
  line( hxs[0], hys[0], hxs[1], hys[1] );
  line( hxs[2], hys[2], hxs[3], hys[3] );
    
  fill( 100 );
  strokeWeight( 1 );
  for ( int idx = 0; idx < 4; ++idx ) {
    if ( which_drag == idx ) {
      stroke( 255, 0, 0 );
    } else {
      stroke( 0, 0, 0 );
    }
    ellipse( hxs[idx], hys[idx], 2.0*radius, 2.0*radius );
  }
}

void mousePressed()
{
  for ( int idx = 0; idx < 4; ++idx ) {
    float dx = mouseX - hxs[idx];
    float dy = mouseY - hys[idx];
    if ( sqrt(sq(dx) + sq(dy)) <= radius ) {
      which_drag = idx;
      break;
    }
  }
}

void mouseDragged()
{
  if ( which_drag != -1 ) {
    hxs[which_drag] = mouseX;
    hys[which_drag] = mouseY;
  }
}

void mouseReleased()
{
  which_drag = -1;
}

