// A simple example of direct manipulation, starring a single
// circular disc.  Demonstrates the main features of direct
// manipulation.

// The model: the current location of the dot.
int handleX;
int handleY;

// Keep track of whether the dot is "live" in the user
// interface.
boolean dragging;

// A useful constant for drawing the dot.
float radius = 20.0;

void setup()
{
  size( 500, 500 );
  handleX = 250;
  handleY = 250;
  dragging = false;
}

void draw()
{
  background( 220 );
  if( dragging ) {
    stroke( 255, 0, 0 );
  } else {
    stroke( 0, 0, 0 );
  }
  ellipse( handleX, handleY, 2.0*radius, 2.0*radius );
}

void mousePressed()
{
  // Here's where we do a hit test: check (via the 
  // Pythagorean theorem) that the current mouse position
  // is inside the dot (by checking the distance from the
  // mouse to the dot's centre).
  if( sqrt(sq(mouseX-handleX) + sq(mouseY-handleY)) <= radius ) {
    // Oh hey, it was inside the dot.  Remember that for 
    // next time there's a mouseDrag event.
    dragging = true;
  }
}

void mouseDragged()
{
  if( dragging ) {
    // If we previously decided that this was a valid
    // drag sequence, update the dot's position to the
    // mouse position.
    
    handleX += mouseX - pmouseX;
    handleY += mouseY - pmouseY;
  }
}

void mouseReleased()
{
  // One more thing: when you let go of the mouse button,
  // we must turn off dragging.  Otherwise the dot will start
  // moving next time you drag, even if the click was outside
  // the dot (try it!).
  dragging = false;
}
