// A more sophisticated demonstration that's roughly the same
// as ThreeButtons.  This example highlights the idea of 
// storing Controllers as global variables, and using those 
// to check who received an event later.  It also shows how
// to assign a custom font to widgets by passing the font in
// at the point where you first initialize ControlP5.

import controlP5.*;

ControlP5 ui;

// Global variables that will refer to the three buttons
// we create.
Button circle_button;
Button rectangle_button;
Button clear_button;

void setup()
{
  size( 400, 300 );

  PFont fnt = createFont( "Helvetica", 18 );
  ui = new ControlP5( this, fnt );
  
  // Once we've got variables referring to the buttons,
  // setting properties on those buttons starts to look
  // a lot more like code we've written in the past.
  
  circle_button = ui.addButton( "Circle" );
  circle_button.setPosition( 10, 10 );
  circle_button.setSize( 120, 30 );
  
  rectangle_button = ui.addButton( "Rectangle" );
  rectangle_button.setPosition( 10, 50 );
  rectangle_button.setSize( 120, 30 );
  
  clear_button = ui.addButton( "Clear" );
  clear_button.setPosition( 10, 90 );
  clear_button.setSize( 120, 30 );
    
  background( 220 );
}

void draw()
{}

void controlEvent( ControlEvent ev )
{
  fill( random(255), random(255), random(255) );

  // Instead of checking which controller got pressed by name,
  // check based on the identity of the Controller object
  // itself.
  if( ev.isFrom( circle_button ) ) {
    float r = lerp( 10, 40, random(1) );
    float x = lerp( 130+r, width-r, random(1) );
    float y = lerp( r, height-r, random(1) );
    ellipse( x, y, 2*r, 2*r );
  } else if( ev.isFrom( rectangle_button ) ) {
    float w = lerp( 10, 60, random(1) );
    float h = lerp( 10, 60, random(1) );
    float x = lerp( 130+w, width-w, random(1) );
    float y = lerp( h, height-h, random(1) );
    rect( x, y, w, h );
  } else if( ev.isFrom( clear_button ) ) {
    background( 220 );
  }
}
